open Format
open Absyn

(* Pretty-printing. Ignore... *)
 

let null (l:'a list) : bool = 
  match l with 
		[] -> true 
	 | _ -> false

let rec iter3 (f:'a -> 'b -> 'c -> unit)  
  (a:'a list) 
  (b:'b list) 
  (c:'c list) : unit = 
  match (a,b,c) with
		(ah::at),(bh::bt), (ch::ct) -> (f ah bh ch; iter3 f at bt ct)
	 | [], [], [] -> ()
	 | _ -> raise (Invalid_argument "iter3: lists must be the same length")
			
  

let rec print_list 
  (exps:'a list)      (* what to print *)
  (pr:'a -> unit)     (* how to print each element *)
  (mid: unit -> unit) (* what to do between each element *)
  : unit = 
  match exps with 
		[] -> ()
	 | [last] -> pr last
	 | (hd :: tl) -> (pr hd;
							mid ();
							print_list tl pr mid)

let string_of_prim (p:prim) :string = 
  (match p with 
		 Plus  -> "+"
	  | Times -> "*"
	  | Minus -> "-"
	  | Equal -> "="
	  | And   -> "and"
	  | Or    -> "or"
	  | Not   -> "not"
	  | Zero  -> "zero?"
	  | LT    -> "<"
	  | GT    -> ">"
	  | Add1  -> "add1"
	  | Sub1  -> "sub1")

let print_prim (p:prim) : unit = 
  print_string (string_of_prim p)

let print_comma _ : unit = 
  (print_string ","; print_cut ())

let rec print_helper (e:exp) : unit = 
  match e with 
    Var s -> print_string s
  | Lambda (ids,e) -> 
      open_box 2;
      print_string "proc (";
		print_list ids print_string print_comma;
      print_string ")";
      print_space();
      print_helper e;
      close_box()
  | App (e1,rands) -> 
      open_box 2;
      print_string "(";
      print_helper e1;
      if not (null rands) then print_space();
      print_list rands print_helper print_space;
      print_string ")";
      close_box()
  | LitInt i -> 
		print_int i
  | LitBool b -> 
		if b then print_string "#t" else print_string "#f"
  | Prim (p,exps) -> 
		open_box 2;
		print_prim p;
		print_string "(";
		print_list exps print_helper print_comma;
		print_string ")";
		close_box()
  | Let (vars, rands, body) -> 
		open_box 0;
		print_string "let ";
		open_vbox 0;
		print_list (List.combine vars rands)
		  (fun (v, exp1) -> 
			  open_box 2;
			  print_string v;
			  print_string " = ";
			  print_helper exp1;
			  close_box ()
		  ) print_cut;
		close_box(); 
		force_newline();
		print_string "in ";
		print_helper body;
		close_box()	
  | If (e1,e2,e3) -> 
		open_box 2;
		print_string "if"; print_space ();
		print_helper e1;
		print_space ();
		print_string "then"; print_space ();
		print_helper e2;
		print_space ();
		print_string "else"; print_space ();
		print_helper e3;
		close_box()
  | Letrec (proc_names, idss, bodies, letrec_body) -> 
		open_box 0;
		print_string "letrec ";
		open_vbox 0;
		print_list 
		  (List.combine proc_names (List.combine idss bodies))
		  (fun (proc_name, (ids, body)) -> 
			  open_box 2;
			  print_string proc_name;
			  print_space ();
			  print_list ids print_string print_space;
			  print_string " = ";
			  print_helper body;
			  close_box ())
		  print_cut;
		close_box ();
		force_newline();
		print_string "in ";
		print_helper letrec_body;
		close_box()

					
(* A function to display an exp on the screen.
   Because its operation is entirely a side effect the 
   result of this function is unit. *)


let print_exp (exp:exp) : unit = 
  open_hbox();
  print_helper exp;
  close_box();
  print_newline()

